	LIST p=16f628a
	; Include file, change directory if needed
	include "P16f628a.inc"

	__CONFIG _XT_OSC & _LVP_OFF & _MCLRE_OFF & _PWRTE_ON & _CP_OFF & _DATA_CP_OFF & _WDT_OFF

;-- -----------------------------------------------
;--  PORTS settings
;-- -----------------------------------------------
	#define RS_PIN PORTA, 2
	#define RW_PIN PORTA, 3
	#define E_PIN  PORTA, 4

;-- MACROS
init_p macro parameter1, parameter2
	movfw parameter2			;-- give W=parameter2
	movwf parameter1			;-- give parameter1=W
	endm

init_v macro parameter, value		;-- macro used to init a variable/register			
	movlw value				;-- give W=value
	movwf parameter				;-- give parameter=W
	endm

	;CBLOCK H'20'	;-- start to declare variables at address H'20'
	;ENDC

;    ,      H'20'
dat0        UDATA	H'20'
;-- variables for interrupt handler, dalay procedures
w_temp 		RES		1 ; variable used for context saving
status_temp RES		1 ; variable used for context saving
DELAY_CNT	RES		1 ; must be initialized before CALL Delay
CntL		RES		1
CntM		RES		1
CntH		RES		1
;-- variables used with LCD 
IS_LCD_ON	RES		1 ; 0 - Off, 1 - On.     LCD
CTR			RES		1

.reset		CODE 0x000
		GOTO	Start

	; Start application beyond vector area

.interrupt	CODE 0x004
		GOTO	ISR
	ISR
		MOVWF	w_temp 			; save W and STATUS
		MOVF 	STATUS,W
		MOVWF 	status_temp
		; isr code goes here
		; isr code goes here
		MOVF	status_temp,W	; restore pre-isr STATUS register
		MOVWF	STATUS
		SWAPF	w_temp,F		; restore pre-isr W register contents
		SWAPF	w_temp,W		; ... without affecting the zero flag
	RETFIE 						; return from interrupt

.program	CODE

; ===========================================
; -
; ===========================================
;   4*(DELAY_CNT+1)   5*(DELAY_CNT+1)  (  4 1   1 )
Delay
		NOP					; 1
		DECFSZ	DELAY_CNT,1	; 1(2) 	
		GOTO	Delay		; 2
	RETURN					; 2

;   ~750..1
BigDelay
		MOVLW	D'3'
		MOVWF	CntH
	LoopH
		MOVLW	D'250'
		MOVWF	CntM
	LoopM
		MOVLW	D'255'
		MOVWF	CntL
	LoopL
		NOP
		DECFSZ	CntL,1
		GOTO	LoopL
		DECFSZ	CntM,1
		GOTO	LoopM
		DECFSZ	CntH,1
		GOTO	LoopH
	RETURN

;    DS18B20.    
;-- ------------------------------------------------------
;--  Project: 	DIGITAL CLOCK, DATE, THERMOMETER 
;-- 				WITH PIC16F84(A)/PIC16F628(A)/PIC16F690 AND DS18S20/DS18B20
;--				+ (in the future: ALARM, THERMOSTAT with DS1631 but only for F690)
;--  Electric diagram & Software: Bogdan Borca/www.bogdi.ro, Timisoara, RO
;--  -----------------------------------------------------
; DS18SB20.inc

	;-- -----------------------------------------------
	;--  THERMOMETER DS18SB20 OPERATIONS 1 WIRE
	;-- -----------------------------------------------

DQ12_Input ;MACRO
		BSF	STATUS, RP0	; select BANK 1
		BSF TRISA, 0	; Make PORTA[0] an input
		BSF TRISA, 1	; Make PORTA[1] an input
		BCF	STATUS, RP0	; select BANK 0
	RETURN	;       
	;ENDM

DQ12_Output ;MACRO
		BSF STATUS, RP0
		BCF TRISA, 0	; Make PORTA[0] an output
		BCF TRISA, 1	; Make PORTA[1] an output
		BCF	STATUS, RP0
	RETURN	;       
	;ENDM

; ===========================================
;    - LED 
;    RS_PIN, RW_PIN  E_PIN
; 8     PORTB
; ===========================================
LDC_BUSY_WAIT
		BSF		RW_PIN
		BSF		STATUS, RP0			; select bank 1
		COMF 	TRISB,1				; Make PORTB all inputs (TRISB=H'FF')
		BCF		STATUS, RP0			; back to bank 0

		;while(PORTB[7]==1) wait
	LoopBusy
		BCF		E_PIN
		NOP
		BSF		E_PIN		
		BTFSC	PORTB,7
		GOTO	LoopBusy
		BCF		E_PIN

		BSF		STATUS, RP0			; select bank 1
		COMF 	TRISB,1				; Make PORTB all outputs (TRISB=H'00')
		BCF		STATUS, RP0			; back to bank 0		
		BCF		RW_PIN
	RETURN

LCD_SEND_CMD	; Sends data stored in W register to LCD
		MOVWF	PORTB
		BSF		E_PIN
		NOP
		BCF		E_PIN
		init_v	DELAY_CNT,D'11'
		CALL	Delay				; must wait 43us as a minimum
	RETURN

LCD_SEND_DATA	; Sends data stored in W register to LCD
		BSF		RS_PIN
		MOVWF	PORTB
		BSF		E_PIN
		NOP
		BCF		E_PIN
		BCF		RS_PIN
		init_v	DELAY_CNT,D'11'
		CALL	Delay				; must wait 43us as a minimum
	RETURN

LCD_CLEAR
		MOVLW	B'00000001'		; Display clear
		CALL	LCD_SEND_CMD
		CALL	LDC_BUSY_WAIT	; wait 1.5 ms or until BF (BusyFlag) is set
	RETURN

LCD_INIT_HD
;15 ms
;B'00110000'
;5 ms
;B'00110000'
;0.1ms
;B'00110000'
;busy?
;B'0011NF00'	function set ()
;busy?
;B'00001000'	display off ()
;busy?
;B'00000001'	display clear ()
;busy?
;B'000001IS'	entyr set mode ()
		; 15ms delay is hold already
		MOVLW	B'00110000'		; H'30'
		CALL	LCD_SEND_CMD
		; >4.1 ms dealy:
		init_v	DELAY_CNT,D'255'
		CALL	Delay
		init_v	DELAY_CNT,D'255'
		CALL	Delay
		init_v	DELAY_CNT,D'255'
		CALL	Delay
		init_v	DELAY_CNT,D'255'
		CALL	Delay
		init_v	DELAY_CNT,D'255'
		CALL	Delay

		MOVLW	B'00110000'		; H'30'
		CALL	LCD_SEND_CMD
		init_v	DELAY_CNT,D'25'
		CALL	Delay			; 100us
		MOVLW	B'00110000'		; H'30'
		CALL	LCD_SEND_CMD

		MOVLW	B'00111100'		; Function Set: 8-bit, 2-line, 5x11 dots
		CALL	LCD_SEND_CMD
		MOVLW	B'00001000'		; Display OFF, Cursor OFF, Blink OFF
		CALL	LCD_SEND_CMD
		CALL	LCD_CLEAR		; B'00000001' & wait 1.5ms
		MOVLW	B'00000110'		; Entry mode set: Increment, No shift
		CALL	LCD_SEND_CMD
		MOVLW	B'00001100'		; Display ON, Cursor OFF, Blink OFF
		CALL	LCD_SEND_CMD

		BSF		IS_LCD_ON,0		; IS_LCD_ON=1, i.e. LCD is initialized
	RETURN

LCD_INIT_KS
		; 30ms delay is hold already
		MOVLW	B'00111100'		; Function Set: 8-bit, 2-line, 5x11 dots
		CALL	LCD_SEND_CMD	;       44 

		MOVLW	B'00001100'		; Display ON, Cursor OFF, Blink OFF
		CALL	LCD_SEND_CMD

		CALL	LCD_CLEAR		; B'00000001' & wait while busy

		MOVLW	B'00000110'		; Entry mode set: Increment, No shift
		CALL	LCD_SEND_CMD

		BSF		IS_LCD_ON,0		; IS_LCD_ON=1, i.e. LCD is initialized
	RETURN

LCD_SHOW_T1	; Displays temperature 1
		MOVLW	H'30'			; "0"
		CALL	LCD_SEND_DATA
		MOVLW	H'31'			; "1"
		CALL	LCD_SEND_DATA
		MOVLW	H'32'			; "2"
		CALL	LCD_SEND_DATA
		MOVLW	H'33'			; "3"
		CALL	LCD_SEND_DATA
		MOVLW	H'34'			; "4"
		CALL	LCD_SEND_DATA
		MOVLW	H'35'			; "5"
		CALL	LCD_SEND_DATA
		MOVLW	H'36'			; "6"
		CALL	LCD_SEND_DATA
		MOVLW	H'37'			; "7"
		CALL	LCD_SEND_DATA
		MOVLW	H'38'			; "8"
		CALL	LCD_SEND_DATA
		MOVLW	H'39'			; "9"
		CALL	LCD_SEND_DATA
	RETURN

LCD_SHOW_T2	; Displays temperature 2
		MOVLW	B'11000000'		; DDRAM Address = 40H (beginning of 2nd line)
		CALL	LCD_SEND_CMD		
		CALL	LCD_SHOW_T1
	RETURN

; ===========================================
;    ()
; ===========================================
Start
		;config registers for input/output
		;CLRF    PORTA			;      
		BSF		STATUS, RP0			; select bank 1
		CLRF 	TRISB				; Make PORTB all outputs
		init_v	TRISA, B'00000011'	; Make PORTA outputs/inputs
		BCF		STATUS, RP0			; back to bank 0
		CLRF	PORTB		;   
							; CMCON setup
		MOVLW  	0x07		; Turn Comparator off and
		MOVWF 	CMCON		; Enable pins for I/O
		; LCD_PowerOn
		BCF		E_PIN			; Disable LCD
		BCF		RS_PIN			;  0
		BCF		RW_PIN			;  0
		CLRF	IS_LCD_ON		; Display is not initialised
main_loop
		CALL	BigDelay		;   750
		; Display values
		BTFSS	IS_LCD_ON,0
			CALL	LCD_INIT_KS		;    ,  
			;CALL	LCD_INIT_HD		;if IS_LCD_ON<>1 then CALL LCD_INIT

		CALL	LCD_CLEAR
		CALL	LCD_SHOW_T1
		CALL	LCD_SHOW_T2
		GOTO	main_loop
	END